package cn.com.qmth.examcloud.exchange.outer.api.provider;

import cn.com.qmth.examcloud.api.commons.EnterpriseService;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.util.HttpClientUtil;
import cn.com.qmth.examcloud.commons.util.ZipUtil;
import cn.com.qmth.examcloud.core.basic.api.StudentCloudService;
import cn.com.qmth.examcloud.core.basic.api.bean.StudentBean;
import cn.com.qmth.examcloud.core.basic.api.request.GetStudentReq;
import cn.com.qmth.examcloud.core.basic.api.response.GetStudentResp;
import cn.com.qmth.examcloud.exchange.outer.service.FaceService;
import cn.com.qmth.examcloud.support.util.IdUtil;
import cn.com.qmth.examcloud.web.config.SystemProperties;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.fileupload.disk.DiskFileItem;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.commons.CommonsMultipartFile;

import java.io.File;
import java.io.IOException;
import java.util.List;
import java.util.Map;

/**
 * 人脸服务
 *
 * @author WANGWEI
 * @date 2018年11月14日
 */
@Api(tags = "学生照片相关接口")
@RestController
@RequestMapping("${$rmp.cloud.exchange.outer}/face")
public class FaceOuterServiceProvider extends ControllerSupport implements EnterpriseService {

    private static final Logger log = LoggerFactory.getLogger(FaceOuterServiceProvider.class);

    private static final long serialVersionUID = -6268268043341270752L;

    /**
     * ZIP最大50M
     */
    private static final int ZIP_MAX_SIZE = 50;

    /**
     * 每张照片最大尺寸 500KB
     */
    private static final int MAX_SIZE = 500;

    @Autowired
    private SystemProperties systemConfig;

    @Autowired
    private FaceService faceService;

    @Autowired
    private StudentCloudService studentCloudService;

    @ApiOperation(value = "获取学生照片URL", hidden = true)
    @PostMapping("getPhotoUrl")
    public String getPhotoUrl(
            @RequestParam @ApiParam(value = "顶级机构ID", example = "123") Long rootOrgId,
            @RequestParam @ApiParam(value = "学生证件号") String identityNumber,
            @RequestParam @ApiParam(value = "学号") String studentCode) {
        if (rootOrgId == null) {
            throw new StatusException("参数rootOrgId的值不能为空！");
        }
        if (StringUtils.isBlank(identityNumber) || StringUtils.isBlank(studentCode)) {
            throw new StatusException("参数identityNumber和studentCode的值不能同时为空，至少传一个值！");
        }
        if (!getEnterpriseRootOrgId().equals(rootOrgId)) {
            throw new StatusException("1000001", "参数rootOrgId的值无效！");
        }

        GetStudentReq req = new GetStudentReq();
        req.setRootOrgId(rootOrgId);
        req.setIdentityNumber(identityNumber);
        req.setStudentCode(studentCode);
        GetStudentResp resp = studentCloudService.getStudent(req);
        StudentBean student = resp.getStudentInfo();
        return student.getPhotoPath();
    }

    @ApiOperation(value = "保存学生照片（图片URL）")
    @PostMapping("addPhotoByUrl")
    public void addPhotoByUrl(
            @RequestParam @ApiParam(value = "顶级机构ID", example = "123", required = true) Long rootOrgId,
            @RequestParam @ApiParam(value = "学生证件号", required = true) String identityNumber,
            @RequestParam @ApiParam(value = "操作者", required = true) String operator,
            @RequestParam @ApiParam(value = "学生照片URL", example = "http://xxx.xxx/xxx.jpg", required = true) String photoUrl) {

        if (!getEnterpriseRootOrgId().equals(rootOrgId)) {
            throw new StatusException("1000001", "rootOrgId is wrong");
        }

        if (photoUrl.startsWith("http")) {
            int lastIndexOf = photoUrl.lastIndexOf(".");
            if (0 > lastIndexOf) {
                throw new StatusException("100002", "参数photoUrl的值需要包含图片后缀名，如../xxx.jpg");
            }

            String fileSuffix = photoUrl.substring(lastIndexOf);
            String fileName = IdUtil.uuid() + fileSuffix;
            File temp = new File(
                    systemConfig.getTempDataDir() + File.separator + "student_photo" + File.separator + fileName);

            try {
                // 文件大小限制
                byte[] bs = HttpClientUtil.get(photoUrl, MAX_SIZE * 1024);

                FileUtils.writeByteArrayToFile(temp, bs);
            } catch (IOException e) {
                throw new StatusException("100003", "文件读写失败");
            }

            faceService.processFace(rootOrgId, identityNumber, fileSuffix, temp, operator, false);
        }
    }

    @ApiOperation(value = "保存学生照片（图片文件）")
    @PostMapping("addPhoto")
    public void addPhoto(@RequestParam @ApiParam(value = "顶级机构ID", example = "123", required = true) Long rootOrgId,
                         @RequestParam @ApiParam(value = "学生证件号", required = true) String identityNumber,
                         @RequestParam @ApiParam(value = "操作者", required = true) String operator,
                         @RequestParam @ApiParam(value = "学生照片文件（文件名称=证件号+图片后缀）", required = true) CommonsMultipartFile file) {

        if (!getEnterpriseRootOrgId().equals(rootOrgId)) {
            throw new StatusException("1000001", "rootOrgId is wrong");
        }

        DiskFileItem item = (DiskFileItem) file.getFileItem();
        File storeLocation = item.getStoreLocation();
        String fileName = file.getOriginalFilename();

        if (StringUtils.containsWhitespace(fileName)) {
            throw new StatusException("600100", "文件名不能含有空白字符");
        }

        if (!fileName.matches("[^\\.\\s]+\\.[^\\.\\s]+")) {
            throw new StatusException("600101", "文件名不合法");
        }

        // 文件大小限制
        if (file.getSize() > MAX_SIZE * 1024) {
            throw new StatusException("620005", "图片大小超过500KB");
        }

        String fileSuffix = fileName.substring(fileName.lastIndexOf("."));

        faceService.processFace(rootOrgId, identityNumber, fileSuffix, storeLocation, operator, false);
    }

    @ApiOperation(value = "保存学生照片（批量图片文件）")
    @PostMapping("importPhotos")
    public List<Map<String, String>> importPhotos(
            @RequestParam @ApiParam(value = "顶级机构ID", example = "123", required = true) Long rootOrgId,
            @RequestParam @ApiParam(value = "操作者", required = true) String operator,
            @RequestParam @ApiParam(value = "学生照片压缩包zip文件，注意压缩包内不要建文件夹（文件名称=证件号+图片后缀）", required = true) CommonsMultipartFile file) {

        if (!getEnterpriseRootOrgId().equals(rootOrgId)) {
            throw new StatusException("1000001", "rootOrgId is wrong");
        }

        DiskFileItem item = (DiskFileItem) file.getFileItem();
        File storeLocation = item.getStoreLocation();
        String filename = file.getOriginalFilename();

        if (!filename.endsWith(".zip")) {
            throw new StatusException("620001", "文件格式不正确，必须是zip格式的压缩包");
        }

        // 文件大小限制
        if (file.getSize() > ZIP_MAX_SIZE * 1024 * 1024) {
            throw new StatusException("620002", "文件大小超过50M,请分批导入");
        }

        List<Map<String, String>> ret = Lists.newArrayList();

        String tempDirPath = systemConfig.getTempDataDir() + File.separator + "photo_import" + File.separator
                + IdUtil.uuid();
        File tempDir = new File(tempDirPath);
        try {
            ZipUtil.unzip(storeLocation, new File(tempDirPath));
        } catch (Exception e) {
            throw new StatusException("620003", "zip文件损坏");
        }

        File[] files = tempDir.listFiles();

        if (null == files) {
            throw new StatusException("620004", "zip文件为空");
        }

        for (File f : files) {
            Map<String, String> map = Maps.newHashMap();
            String fileName = f.getName();
            map.put("file", fileName);

            if (f.length() > MAX_SIZE * 1024) {
                map.put("statusCode", "EX-620005");
                map.put("statusDesc", "文件大小超过500KB");
                ret.add(map);
                continue;
            }

            try {
                if (StringUtils.containsWhitespace(fileName)) {
                    throw new StatusException("600100", "文件名不能含有空白字符");
                }

                if (!fileName.matches("[^\\.\\s]+\\.[^\\.\\s]+")) {
                    throw new StatusException("600101", "文件名不合法");
                }
                String identityNumber = fileName.substring(0, fileName.lastIndexOf("."));
                String fileSuffix = fileName.substring(fileName.lastIndexOf("."));

                faceService.processFace(rootOrgId, identityNumber, fileSuffix, f, operator, false);

                map.put("statusCode", "200");
                map.put("statusDesc", "成功");

            } catch (StatusException e) {
                map.put("statusCode", e.getCode());
                map.put("statusDesc", e.getDesc());
            } catch (Exception e) {
                map.put("statusCode", "系统异常");
                map.put("statusDesc", e.getCause().getMessage());
            }
            ret.add(map);
        }

        try {
            FileUtils.deleteDirectory(tempDir);
        } catch (IOException e) {
            LOGGER.error("fail to clean temp dir.", e);
        }

        return ret;
    }

}
